@@ -12,6 +12,7 @@ from keras_cv_attention_models.halonet.halonet import (
     HaloNet50T,
     HaloNetSE33T,
     HaloNextECA26T,
+    HaloRegNetZB,
     halo_attention,
 )
 
@@ -70,6 +71,7 @@ Model architectures:
   | HaloNetSE33T   | 13.7M  | 256              | 80.99    |
   | HaloNextECA26T | 10.7M  | 256              | 78.84    |
   | HaloNet50T     | 22.7M  | 256              | 81.5 ?   |
+  | HaloRegNetZB   | 11.68M | 224              | 81.058   |
 """
 
 HaloNetH0.__doc__ = __head_doc__ + """
@@ -91,6 +93,7 @@ Args:
 HaloNetSE33T.__doc__ = HaloNet26T.__doc__
 HaloNextECA26T.__doc__ = HaloNet26T.__doc__
 HaloNet50T.__doc__ = HaloNet26T.__doc__
+HaloRegNetZB.__doc__ = HaloNet26T.__doc__
 
 halo_attention.__doc__ = __head_doc__ + """
 Halo Attention. Defined as function, not layer.
